class GroundPicture < ActiveRecord::Base
  belongs_to :panel
  belongs_to :picture
  
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :repeat, :numericality => true, :inclusion => { :in => 0..3 }
  validates :x, :numericality => true
  validates :y, :numericality => true
  validates :picture_id, :numericality => true, :existence => {:both => false}
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  
  def supply_default
    self.repeat = 0
    self.x = 0
    self.y = 0
  end
  
  def overwrite
  end
  
  def visible? au
    if au == nil
      return false if MagicNumber['run_mode'] == 1
    elsif au.is_a?(Author)
    elsif au.is_a?(Admin)
      return true
    else
      return false
    end
    self.panel.publish?
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(GroundPicture.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => 'panels.publish > 0', :order => 'ground_pictures.updated_at desc'})
    GroundPicture.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:panel => {:author => {}}, :picture => {:artist => {}, :license => {}}}}
  end
  
  def self.list_json_opt
    {:include => {:panel => {:include => {:author => {}}}, :picture => {:include => {:artist => {}, :license => {}}} }}
  end
  
  def self.mylist au, page = 1, page_size = Author.default_ground_picture_page_size
    opt = {}
    opt.merge!(self.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['panels.author_id = ?', au.id], :order => 'ground_pictures.updated_at desc'})
    GroundPicture.find(:all, opt)
  end
  
  def self.show cid, au
    opt = {}
    opt.merge!(GroundPicture.show_opt)
    res = GroundPicture.find(cid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(au)
    res
  end
  
  def self.show_opt
    {:include => {:panel => {:author => {}}, :picture => {:artist => {}, :license => {}}}}
  end
  
  def self.show_json_opt
    {:include => {:panel => {:include => {:author => {}}}, :picture => {:include => {:artist => {}, :license => {}}} }}
  end
  
end
