# -*- encoding: utf-8 -*-
#借手
require 'spec_helper'

describe Demander do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = @user.author    #ユーザ作成時に連動して作成される
    @demand_user = FactoryGirl.create :demand_user
  end
  describe '検証に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.build :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @demander.name = 'a'
        @demander.caption = 'a'
        @demander.url = 'http://test.jp/'
        @demander.description = 'a'
        @demander.demander_url = 'http://test.jp/'
        @demander.should be_valid
      end
      it '上限データが通る' do
        @demander.name = 'a'*50
        @demander.caption = 'a'*30
        @demander.url = 'http://test.jp/aaaaa' + 'a' * 180
        @demander.description = 'a' * 99999
        @demander.demander_url = 'http://test.jp/aaaaa' + 'a' * 180
        @demander.should be_valid
      end
    end
    
    context 'demand_user_idを検証するとき' do
      it 'nullなら失敗する' do
        @demander.demand_user_id = nil
        @demander.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @demander.demand_user_id = 'a'
        @demander.should_not be_valid
      end
      it '存在する借手ユーザでなければ失敗する' do
        @demander.demand_user_id = 0
        @demander.should_not be_valid
      end
    end
    context 'nameを検証するとき' do
      it 'nullなら失敗する' do
        @demander.name = ''
        @demander.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @demander.name = 'a'*51
        @demander.should_not be_valid
      end
      it '重複していたら失敗する' do
        l = FactoryGirl.create :demander
        @demander.should_not be_valid
      end
    end
    context 'captionを検証するとき' do
      it 'nullなら失敗する' do
        @demander.caption = ''
        @demander.should_not be_valid
      end
      it '31文字以上なら失敗する' do
        @demander.caption = 'a'*31
        @demander.should_not be_valid
      end
    end
    context 'urlを検証するとき' do
      it 'nullなら失敗する' do
        @demander.url = ''
        @demander.should_not be_valid
      end
      it '201文字以上なら失敗する' do
        @demander.url = 'http://test.jp/aaaaa' + 'a' * 181
        @demander.should_not be_valid
      end
      it 'url形式でないなら失敗する' do
        @demander.url = 'aaaaaaa'
        @demander.should_not be_valid
      end
    end
    context 'descriptionを検証するとき' do
      it 'nullなら失敗する' do
        @demander.description= ''
        @demander.should_not be_valid
      end
    end
    context 'demander_urlを検証するとき' do
      it 'nullなら失敗する' do
        @demander.demander_url = ''
        @demander.should_not be_valid
      end
      it '201文字以上なら失敗する' do
        @demander.demander_url = 'http://test.jp/aaaaa' + 'a' * 181
        @demander.should_not be_valid
      end
      it 'url形式でないなら失敗する' do
        @demander.demander_url = 'aaaaaaa'
        @demander.should_not be_valid
      end
    end
    context 'demander_status_idを検証するとき' do
      it 'nullなら失敗する' do
        @demander.demander_status_id = nil
        @demander.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @demander.demander_status_id = 'a'
        @demander.should_not be_valid
      end
      it '存在する借受状況でなければ失敗する' do
        @demander.demander_status_id = 0
        @demander.should_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it 'defined' do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.build :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
      @demander.supply_default
    end
  end
  
  describe '上書き補充に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.build :demander, :demander_status_id => @ds.id, :demand_user_id => nil
    end
    it 'demand_user_idに借手アカウントidを補充している' do
      @demander.overwrite @demand_user
      @demander.demand_user_id.should_not be_nil
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.build :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    it 'パラメータが借手アカウントでないならno' do
      @demander.own?(nil).should == false
    end
    it '借手アカウントのidと一致するならyes' do
      @demander.own?(@demand_user).should == true
    end
    it '借手アカウントのidと一致しないならno' do
      @other_demand_user = FactoryGirl.create :demand_user
      @demander.own?(@other_demand_user).should == false
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.build :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    it 'パラメータが借手アカウントでないならno' do
      @demander.visible?(nil).should == false
    end
    it '借手アカウントのidと一致するならyes' do
      @demander.visible?(@demand_user).should == true
    end
    it '借手アカウントのidと一致しないならno' do
      @other_demand_user = FactoryGirl.create :demand_user
      @demander.visible?(@other_demand_user).should == false
    end
  end
  
  describe '状態に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    it '貸与状況の状態に問い合わせしている' do
      DemanderStatus.any_instance.stub(:status).with(any_args).and_return(1)
      DemanderStatus.any_instance.should_receive(:status).with(any_args).exactly(1)
      r = @demander.status
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id, :name => "6"
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Demander.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Demander.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Demander.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Demander.page_size('7').should eq 7
      end
      it 'nilの場合はDemander.default_page_sizeになる' do
        Demander.page_size().should eq Demander.default_page_size
      end
      it '0以下の場合はDemander.default_page_sizeになる' do
        Demander.page_size('0').should eq Demander.default_page_size
      end
      it 'Demander.max_page_sizeを超えた場合はDemander.max_page_sizeになる' do
        Demander.page_size('1000').should eq Demander.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Demander.stub(:list_opt).with(any_args).and_return({})
        Demander.should_receive(:list_opt).with(any_args).exactly(1)
        r = Demander.list
      end
    end
    it 'リストを返す' do
      r = Demander.list
      r.should eq [@demander]
    end
    it '管理名で並んでいる' do
      v = FactoryGirl.create :demander, :name => "0"
      r = Demander.list
      r.should eq [v, @demander]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ds2 = FactoryGirl.create :demander_status
        @demander2 = FactoryGirl.create :demander, :demander_status_id => @ds2.id, :demand_user_id => @demand_user.id, :name => "5"
        @ds3 = FactoryGirl.create :demander_status
        @demander3 = FactoryGirl.create :demander, :demander_status_id => @ds3.id, :demand_user_id => @demand_user.id, :name => "4"
        @ds4 = FactoryGirl.create :demander_status
        @demander4 = FactoryGirl.create :demander, :demander_status_id => @ds4.id, :demand_user_id => @demand_user.id, :name => "3"
        @ds5 = FactoryGirl.create :demander_status
        @demander5 = FactoryGirl.create :demander, :demander_status_id => @ds5.id, :demand_user_id => @demand_user.id, :name => "2"
        Demander.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        r = Demander.list
        r.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #管理名で並んでいる
        r = Demander.list(1)
        r.should eq [@demander5, @demander4]
      end
      it 'page=2なら中間2件を返す' do
        r = Demander.list(2)
        r.should eq [@demander3, @demander2]
      end
      it 'page=3なら先頭1件を返す' do
        r = Demander.list(3)
        r.should eq [@demander]
      end
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @ds2 = FactoryGirl.create :demander_status
        @demander2 = FactoryGirl.create :demander, :demander_status_id => @ds2.id, :demand_user_id => @demand_user.id, :name => "5"
        @ds3 = FactoryGirl.create :demander_status
        @demander3 = FactoryGirl.create :demander, :demander_status_id => @ds3.id, :demand_user_id => @demand_user.id, :name => "4"
        @ds4 = FactoryGirl.create :demander_status
        @demander4 = FactoryGirl.create :demander, :demander_status_id => @ds4.id, :demand_user_id => @demand_user.id, :name => "3"
        @ds5 = FactoryGirl.create :demander_status
        @demander5 = FactoryGirl.create :demander, :demander_status_id => @ds5.id, :demand_user_id => @demand_user.id, :name => "2"
        Demander.stub(:default_page_size).and_return(2)
      end
      it '件数0は全件(5件)を返す' do
        r = Demander.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '待機中一覧取得に於いて' do
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Demander.list_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = Demander.list_opt[:include]
      r.should have(1).items
    end
    it '貸与状況を含んでいる' do
      r = Demander.list_opt[:include]
      r.has_key?(:demander_status).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    it '貸与状況を含んでいる' do
      r = Demander.list.to_json Demander.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('demander_status').should be_true
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Demander.stub(:show_opt).with(any_args).and_return({})
        Demander.should_receive(:show_opt).with(any_args).exactly(1)
        r = Demander.show @demander.id, @demand_user
      end
      it '閲覧許可を問い合わせている' do
        Demander.any_instance.stub(:visible?).with(any_args).and_return(true)
        Demander.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = Demander.show @demander.id, @demand_user
      end
    end
    it '指定の借手を返す' do
      r = Demander.show @demander.id, @demand_user
      r.should eq @demander
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        Demander.any_instance.stub(:visible?).and_return(false)
        lambda{
          Demander.show @demander.id, @demand_user
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない借手を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Demander.show 110, @demand_user
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  
  describe '編集取得に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Demander.stub(:show_opt).with(any_args).and_return({})
        Demander.should_receive(:show_opt).with(any_args).exactly(1)
        r = Demander.edit @demander.id, @demand_user
      end
      it '所持判定を問い合わせている' do
        Demander.any_instance.stub(:own?).with(any_args).and_return(true)
        Demander.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = Demander.edit @demander.id, @demand_user
      end
    end
    it '指定の借手を返す' do
      Demander.any_instance.stub(:own?).and_return(true)
      r = Demander.edit @demander.id, @demand_user
      r.should eq @demander
    end
    context '権限がなかったとき' do
      it '403Forbidden例外を返す' do
        Demander.any_instance.stub(:own?).and_return(false)
        lambda{
          Demander.edit @demander.id, @demand_user
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない借手を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Demander.edit 110, @demand_user
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Demander.show_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = Demander.show_opt[:include]
      r.should have(1).items
    end
    it '貸与状況を含んでいる' do
      r = Demander.show_opt[:include]
      r.has_key?(:demander_status).should be_true
    end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @ds = FactoryGirl.create :demander_status
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
    end
    it '貸与状況を含んでいる' do
      r = Demander.show(@demander.id, @demand_user).to_json Demander.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('demander_status').should be_true
    end
  end
  
  describe '更新作成に於いて' do
    before do
      DemanderStatus.any_instance.stub(:supply_default).with(any_args)
    end
    context 'つつがなく終わるとき' do
      before do
        @demander = FactoryGirl.build :demander, :demander_status_id => nil, :demand_user_id => @demand_user.id
      end
      it '借手が追加される' do
        lambda {
          r = @demander.store
        }.should change Demander, :count
      end
      it '貸与状況が作成される' do
        lambda {
          r = @demander.store
        }.should change DemanderStatus, :count
      end
      it 'trueを返す' do
        r = @demander.store
        r.should be_true
      end
    end
    context '貸与状況があるとき' do
      before do
        @ds = FactoryGirl.create :demander_status
        @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id, :demand_user_id => @demand_user.id
      end
      it '借手は変化しない' do
        lambda {
          r = @demander.store
        }.should_not change Demander, :count
      end
      it '貸与状況は変化しない' do
        lambda {
          r = @demander.store
        }.should_not change DemanderStatus, :count
      end
    end
    context '貸与状況の作成に失敗したとき' do
      before do
        DemanderStatus.any_instance.stub(:save).with(any_args).and_return(false)
        @demander = FactoryGirl.build :demander, :demander_status_id => nil, :demand_user_id => @demand_user.id
      end
      it '借手は変化しない' do
        lambda {
          r = @demander.store
        }.should_not change Demander, :count
      end
      it '貸与状況は変化しない' do
        lambda {
          r = @demander.store
        }.should_not change DemanderStatus, :count
      end
      it '借手の検証は失敗している' do
        r = @demander.store
        r.should be_false
      end
    end
  end
  
  describe '借受停止に於いて' do
    before do
      @ds = FactoryGirl.create :demander_status, :requested_at => Time.now, :approved_at => Time.now, :rejected_at => Time.now, :receive_hour1 => 0, :receive_hour2 => 0
      @demander = FactoryGirl.create :demander, :demander_status_id => @ds.id
    end
    context '事前チェックする' do
      it '貸与状況に借受停止を依頼している' do
        DemanderStatus.any_instance.stub(:stop).with(any_args).and_return(true)
        DemanderStatus.any_instance.should_receive(:stop).with(any_args).exactly(1)
        r = @demander.stop
      end
    end
    context 'つつがなく終わるとき' do
      it 'Trueを返す' do
        DemanderStatus.any_instance.stub(:stop).with(any_args).and_return(true)
        r = @demander.stop
        r.should be_true
      end
    end
  end
end
